@@ -2,14 +2,13 @@
 // #![warn(missing_docs)]
 
 pub mod body;
-pub mod rejection;
+mod redirect;
 mod request;
 pub mod response;
 mod result_ext;
 pub mod route;
-pub mod service;
 
-use std::convert::TryFrom;
+use std::convert::{Infallible, TryFrom};
 use std::future::Future;
 use std::pin::Pin;
 use std::task::{Context, Poll};
@@ -20,8 +19,8 @@ pub use http;
 pub use hyper;
 use hyper::body::HttpBody;
 use hyper::service::Service;
-use rejection::Rejection;
-pub use request::{Request, RequestExt};
+pub use redirect::Redirect;
+pub use request::{DecodeQueryError, Request, RequestExt};
 pub use response::{IntoResponse, Response};
 pub use result_ext::ResultExt;
 
@@ -36,13 +35,15 @@ impl<S, H> SolarSail<S, H> {
         SolarSail { state, handler }
     }
 
-    async fn serve<F>(&self, req: Request) -> Result<http::Response<hyper::Body>, Rejection>
+    async fn serve<F>(&self, req: Request) -> http::Response<hyper::Body>
     where
         S: Clone,
         H: Fn(S, Request) -> F,
-        F: Future<Output = Result<Response, Rejection>>,
+        F: Future<Output = Response>,
     {
-        (self.handler)(self.state.clone(), req).await
+        (self.handler)(self.state.clone(), req)
+            .await
+            .into_response()
     }
 
     #[cfg(feature = "server")]
@@ -50,7 +51,7 @@ impl<S, H> SolarSail<S, H> {
     where
         S: Clone + Send + Sync + 'static,
         H: Fn(S, Request) -> F + Clone + Send + Sync + 'static,
-        F: Future<Output = Result<Response, Rejection>> + Send + 'static,
+        F: Future<Output = Response> + Send + 'static,
     {
         let server = hyper::Server::bind(addr).serve(self);
         println!("Listening on http://{}", addr);
@@ -66,7 +67,7 @@ impl<S, H> SolarSail<S, H> {
     where
         S: Clone + Send + Sync + 'static,
         H: Fn(S, Request) -> F + Clone + Send + Sync + 'static,
-        F: Future<Output = Result<Response, Rejection>> + Send + 'static,
+        F: Future<Output = Response> + Send + 'static,
         L: tower_layer::Layer<Self>,
         L::Service: Service<http::Request<hyper::Body>, Response = http::Response<B>>
             + Clone
@@ -96,10 +97,10 @@ impl<S, H, F> Service<http::Request<hyper::Body>> for SolarSail<S, H>
 where
     S: Clone + Send + Sync + 'static,
     H: Fn(S, Request) -> F + Clone + Send + Sync + 'static,
-    F: Future<Output = Result<Response, Rejection>> + Send + 'static,
+    F: Future<Output = Response> + Send + 'static,
 {
     type Response = http::Response<hyper::Body>;
-    type Error = Rejection;
+    type Error = Infallible;
     // TODO: get rid of Box?
     type Future = BoxTrySendFuture<Self::Response, Self::Error>;
 
@@ -124,7 +125,7 @@ where
 
         let svc = self.clone();
         let req = Request::from_parts(parts, body);
-        Box::pin(async move { svc.serve(req).await })
+        Box::pin(async move { Ok(svc.serve(req).await) })
     }
 }
 
