@@ -1,4 +1,3 @@
-use crate::rejection::Rejection;
 use crate::response::Response;
 use crate::IntoResponse;
 
@@ -33,10 +32,9 @@ where
     E: std::error::Error + Send + Sync + 'static + IntoResponse,
     F: FnOnce(Response) -> Response,
 {
-    fn into_response(self) -> Result<Response, Rejection> {
-        let res = self.inner.into_response()?;
-        let res = (self.map)(res);
-        Ok(res)
+    fn into_response(self) -> Response {
+        let res = self.inner.into_response();
+        (self.map)(res)
     }
 }
 
